@@ -81,7 +81,9 @@ impl Entities {
             return Err(CustomErrors::ComponentNotRegistered.into());
         };
 
-        self.map[index] ^= *mask;
+        if self.has_component(index, *mask) {
+            self.map[index] ^= *mask;
+        }
 
         Ok(())
     }
@@ -109,6 +111,10 @@ impl Entities {
         }
         Ok(())
     }
+
+    fn has_component(&self, index: usize, mask: u32) -> bool {
+        self.map[index] & mask == mask
+    }
 }
 
 #[cfg(test)]
@@ -256,6 +262,21 @@ mod test {
         Ok(())
     }
 
+    #[test]
+    fn should_not_add_component_back_after_deleting_twice() -> Result<()> {
+        let mut entities = Entities::default();
+        entities.register_component::<u32>();
+        entities.register_component::<f32>();
+        entities
+            .create_entity()
+            .with_component(100_u32)?
+            .with_component(50.0_f32)?;
+        entities.delete_component_by_entity_id::<u32>(0)?;
+        entities.delete_component_by_entity_id::<u32>(0)?;
+        assert_eq!(entities.map[0], 2);
+        Ok(())
+    }
+
     struct Health(pub u32);
     struct Speed(pub u32);
 }
